#!/bin/sh
~iworx/bin/log.pex --message "Installing Let's Encrypt";
# Get OS version
OS_VER=`cat /etc/redhat-release | cut -d" " -f3 | cut -d "." -f1`
UNINSTALL_EPEL=1
UNINSTALL_IUS=1

if [ $OS_VER -lt 6 ]; then
  ~iworx/bin/log.pex --message "Unsupported OS, exiting";
  exit 1
fi

# Check if repos are already installed
if [ -e "/etc/yum.repos.d/epel.repo" ]; then
  UNINSTALL_EPEL=0
fi

if [ -e "/etc/yum.repos.d/ius.repo" ]; then
  UNINSTALL_IUS=0
fi

# Install necessary repositories and python packages
if [ $OS_VER -eq 6 ]; then
  ~iworx/bin/log.pex --message "Installing EPEL Repo";
  rpm -ivh https://dl.fedoraproject.org/pub/epel/epel-release-latest-6.noarch.rpm > /dev/null 2>&1
  ~iworx/bin/log.pex --message $?
  ~iworx/bin/log.pex --message "Installing IUS Repo";
  rpm -ivh https://rhel6.iuscommunity.org/ius-release.rpm > /dev/null 2>&1
  ~iworx/bin/log.pex --message $?
  ~iworx/bin/log.pex --message "Installing Python27";
  yum -y install python27 python27-devel python27-pip python27-setuptools python27-virtualenv > /dev/null 2>&1
  ~iworx/bin/log.pex --message $?
fi

# Install git
~iworx/bin/log.pex --message "Installing Git"
yum -y install git > /dev/null 2>&1
~iworx/bin/log.pex --message $?

# Create clone directory for git
mkdir /home/interworx/lib/letsencrypt > /dev/null 2>&1

# Clone letsencrypt repository
~iworx/bin/log.pex --message "Cloning git repo";
git clone https://github.com/letsencrypt/letsencrypt /home/interworx/lib/letsencrypt > /dev/null 2>&1
~iworx/bin/log.pex --message $?

# Run LetsEncrypt initialization
if [ ! -e /etc/letsencrypt ]; then
  ~iworx/bin/log.pex --message "Running Let's Encrypt installer";
  /home/interworx/lib/letsencrypt/letsencrypt-auto --register-unsafely-without-email --agree-tos > /dev/null 2>&1
  ~iworx/bin/log.pex --message $?
  ~iworx/bin/log.pex --message "Let's Encrypt apparently returns 1 as the norm";
fi

# Uninstall repos if necessary
if [ $UNINSTALL_EPEL -eq 1 ]; then
  yum -y remove epel-release > /dev/null 2>&1
fi

if [ $UNINSTALL_IUS -eq 1 ]; then
  yum -y remove ius-release > /dev/null 2>&1
fi
