<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Paul Oehler <poehler@interworx.com>
 * @date    Sep 22, 2009
 */

/**
 * R1soft integration plugin.
 *
 * @package    InterWorx
 * @subpackage Plugin
 */
class Plugin_LetsEncrypt extends Plugin {
  private $_last_error = '';

  /**
   * Init the plugin, disabling if necessary.
   */
  protected function _init() {
    if( $this->_checkOS() === false ) {
      $this->_disable( $this->_last_error );
    }
  }

  /**
   * Install the plugin.
   */
  public function install() {
    IWorxLog::log( 'Installing ' . get_class( $this ) );
    $letsencrypt = '/home/interworx/bin/install-letsencrypt.sh';
    IWorxLog::log( "Running {$letsencrypt}" );

    $cmd  = Ini::get( Ini::IWORX_BIN, 'runasuser' );
    $cmd .= " root custom '{$letsencrypt}'";

    IWorxLog::log( $cmd );

    IWorxExec::exec( $cmd, $result, $retval, IWorxExec::STDERR_2_STDOUT );
    IWorxLog::log( $retval );
    IWorxLog::log( $result );
  }

  /**
   * Form validation.
   *
   * @param Form $Form
   * @return boolean
   */
  public function validateMyEditForm( Form $Form ) {
    if( $Form->getValue( 'status' ) === '0' ) {
      return true;
    }
    if( $this->_checkOS() === false ) {
      $Form->addError( $this->_last_error );
    }
    return !$Form->hasErrors();
  }

  /**
   * Determine if the OS is supported by LetsEncrypt.
   *
   * Sets $this->_last_error to a string error message.
   *
   * @return boolean
   */
  private function _checkOS() {
    $distro = IWorxSysInfo::distro();
    $found  = preg_match( '/release ([0-9]+)/', $distro, $matches );
    if( $found === 0 ) {
      $this->_last_error = 'Could not determine OS version';
    } else {
      $version = intval( $matches[1] );
      if( $version <= 5 ) {
        $this->_last_error = "Let's Encrypt is not supported on your OS: {$distro}";
      }
    }

    return $this->_last_error === '';
  }

  /**
   * Get category.
   *
   * @return string
   */
  public function getCategory() {
    return Plugin_Category::SSL;
  }
}
